// RaspiClientConsole.java

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import ch.aplu.util.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;

public class RaspiClientConsole
{
  class StreamReader extends Thread
  {
    private InputStream is;

    public StreamReader(InputStream is)
    {
      this.is = is;
    }

    public void run()
    {
      while (true)
      {
        String response = "";
        try
        {  
          response = readResponse(is);
        }
        catch (Exception ex)
        {
          c.println("Exception in readResponse: " + ex.getMessage());
          break;
        }  
        int rc = 0;
        try
        {
          rc = Integer.parseInt(response);
        }
        catch (NumberFormatException ex)
        {
        }
        if (rc >= 0)
        {
          c.print("Response: " + response);
          c.println(" in " + timer.getTime() / 1000 + " ms");
        }
        else
          c.println("Response (error): " + responseMsg[-rc]);
      }
    }

    private String readResponse(InputStream is) throws IOException
    {
      if (is == null)
        return "";
      byte[] buf = new byte[4096];
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      boolean done = false;
      while (!done)
      {
        int len = is.read(buf);
        if (len == -1)
          throw new IOException("Stream closed");
        baos.write(buf, 0, len);
        if (buf[len - 1] == 10)  // \n
          done = true;
      }
      String s = baos.toString("UTF-8");
      return s.substring(0, s.length() - 1);  // Remove \n
    }

  }

  String[] responseMsg =
  {
    "OK", "SEND_FAILED", "ILLEGAL_METHOD",
    "ILLEGAL_INSTANCE", "CMD_ERROR", "CREATION_FAILED"
  };

  private int port = 1299;
  private OutputStream os = null;
  private InputStream is = null;    
  private HiResTimer timer = new HiResTimer();
  private ch.aplu.util.Console c = new ch.aplu.util.Console();

  public RaspiClientConsole()
  {
    try
    {
      c.print("IP address? ");
      String ipAddress = c.readLine();
      c.println("Trying to connect to " + ipAddress);
      Socket s = new Socket(ipAddress, port);
      c.println("Connection established.");
      c.println("Enter command<cr>");
      os = s.getOutputStream();
      is = s.getInputStream();
      StreamReader sr = new StreamReader(is);
      sr.start();
      String command = "";
      while (true)
      {
        command = c.readLine();
        if (command.length() == 0)
        {
          c.println("Illegal command");
          continue;
        }
        timer.start();
        sendCommand(command);
      }
    }
    catch (Exception ex)
    {
      c.println("Connection failed");
    }
  }

  private void sendCommand(String cmd) throws IOException
  {
    if (cmd == null || cmd.length() == 0 || os == null)
      throw new IOException("sendCommand failed.");
    cmd += "\n";  // Append \n
    byte[] ary = cmd.getBytes(Charset.forName("UTF-8"));
    os.write(ary);
    os.flush();
  }

  public static void main(String[] args)
  {
    new RaspiClientConsole();
  }
}
